﻿using System;
using System.IO;
using System.Net;
using System.Text;
using Newtonsoft.Json;

namespace DisqusSharp
{
    static class Helper
    {
        public static DisqusResponse Get(string url)
        {
            return Get(url, string.Empty);
        }

        public static DisqusResponse Get(string url, string getData)
        {
            if (!string.IsNullOrEmpty(getData))
                url += "?" + getData;

            WebRequest request = WebRequest.Create(url);
            request.Method = "GET";

            Stream dataStream;

            try
            {
                WebResponse response = request.GetResponse();
                dataStream = response.GetResponseStream();
            }
            catch (WebException ex)
            {
                if (((HttpWebResponse)(ex.Response)).StatusCode == HttpStatusCode.BadRequest)
                    dataStream = ex.Response.GetResponseStream();
                else
                    throw new DisqusUnknownException();
            }

            StreamReader reader = new StreamReader(dataStream);

            DisqusResponse disqusResponse = JsonConvert.DeserializeObject<DisqusResponse>(reader.ReadToEnd());

            if (disqusResponse.Succeeded)
                return disqusResponse;

            ThrowError(disqusResponse);

            throw new DisqusUnknownException();
        }

        public static DisqusResponse Post(string url, string postData, Encoding encoder)
        {
            byte[] data = encoder.GetBytes(postData);

            WebRequest request = WebRequest.Create(url);
            request.Method = "POST";
            request.ContentType = "application/x-www-form-urlencoded";
            request.ContentLength = data.Length;

            Stream dataStream = request.GetRequestStream();
            dataStream.Write(data, 0, data.Length);
            dataStream.Close();

            try
            {
                WebResponse response = request.GetResponse();
                dataStream = response.GetResponseStream();
            }
            catch (WebException ex)
            {
                if (((HttpWebResponse)(ex.Response)).StatusCode == HttpStatusCode.BadRequest)
                    dataStream = ex.Response.GetResponseStream();
                else
                    throw new DisqusUnknownException();
            }

            StreamReader reader = new StreamReader(dataStream);

            DisqusResponse disqusResponse = JsonConvert.DeserializeObject<DisqusResponse>(reader.ReadToEnd());

            if (disqusResponse.Succeeded)
                return disqusResponse;

            ThrowError(disqusResponse);

            throw new DisqusUnknownException();
        }

        public static string GenerateQueryString(string key, DisqusPostTypes? value)
        {
            if (value == null)
                return string.Empty;

            if ((value & DisqusPostTypes.None) == DisqusPostTypes.None)
                return string.Empty;


            string strFilter = Disqus.ToString(value.Value);

            if (!string.IsNullOrEmpty(strFilter))
                return string.Format("{0}={1}", key, strFilter);

            return string.Empty;
        }

        #region Error Helpers

        static void ThrowError(DisqusResponse disqusResponse)
        {
            if (disqusResponse.Code == "input-invalid")
                ThrowInputInvalidError(disqusResponse);
            else if (disqusResponse.Code == "key-error")
                ThrowKeyError(disqusResponse);

            throw new NotImplementedException();
        }

        private static void ThrowKeyError(DisqusResponse disqusResponse)
        {
            string message = disqusResponse.Message.ToString();

            switch (message)
            {
                case "API Key does not allow access to this forum.":
                    throw new DisqusApiKeyAccessDeniedException();
                case "No valid API key found":
                    throw new DisqusInvalidApiKeyException();
                default:
                    break;
            }

            throw new NotImplementedException();
        }

        static void ThrowInputInvalidError(DisqusResponse disqusResponse)
        {
            string message = disqusResponse.Message.ToString();

            switch (disqusResponse.Message.ToString())
            {
                case "Invalid user api key.":
                    throw new DisqusInvalidUserApiKeyException();
                default:
                    if (message.StartsWith("Key") && message.EndsWith("could not be converted to UserKey object"))
                        throw new DisqusInvalidUserApiKeyException(message);
                    if (message.StartsWith("id "))
                    {
                        if (message.EndsWith("could not be converted to Forum object"))
                            throw new DisqusInvalidForumIdException(message);
                        if (message.EndsWith("could not be converted to Thread object"))
                            throw new DisqusInvalidThreadIdException(message);
                    }

                    break;
            }

            throw new NotImplementedException();
        }


        #endregion

    }
}
